;#####################################################################
;#                                                                   #
;#                    END2.SCE                                       #
;#                                                                   #
;#  CONTENTS:                                                        #
;#     Parameters:                                                   #
;#         scenarios:  end-scenario                                  #
;#                        steps:  ask-if-all, bring-something-else   #
;#                                present-bill, yes-response         #
;#                     wrap-up                                       #
;#                         steps:  give-bill, say-good-bye           #
;#                                                                   #
;#####################################################################

(defparameter ask-if-all
;; As customer if they want anything else 
  (make-instance 'mystep
		 :name "ask-if-all"
		 :my-condition (spscorec :sc 2 
					 :nec
					 (andc 
					  :nec (list
						(notc 
						 (historyc "present-bill"))
						(notc (historyc "ask-if-all")))
					  ))
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~%~a" (random-pick all))
				(setf (expecting-input (waiter restaurant)) t))
		 ))

(defparameter bring-something-else 
;; if customer answer no to ask-if-all
  (make-instance 'mystep
		 :name "bring-something-else"
		 :input-condition (andc :nec (list
					      (wordc :nec '(no))
					      (notc (wordc 
						     :nec '(orderable-item)))
					      (notc (wordc 
						     :nec '(non-orderable-item)
						     ))))
		 :my-condition (lambdac #'(lambda (restaurant mystep)
					    (prev-step-matches restaurant
							       '("ask-if-all")
							       2)))
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~%What else can I bring for you?")
				(setf (expecting-input (waiter restaurant)) t))
		 ))

(defparameter present-bill 
  (make-instance 'mystep
		 :name "present-bill"
		 :input-condition (wordc :nec '(bill)
					 :sup '(want see have))
		 
		 :my-action #'(lambda (self restaurant)
				
				(say (waiter restaurant) 
				     "~%Here is your bill.")
				(say (waiter restaurant) "~% ")
				(display-bill (order-pad restaurant))
				(say (waiter restaurant) 
				     "~%Your total comes to $~a"
				     (calculate-bill (order-pad restaurant)))
				
				)))

(defparameter yes-response 
  (make-instance 'mystep
		 :name "yes-response"
		 :input-condition (wordc :nec '(yes))
		 :my-condition (lambdac #'(lambda (restaurant mystep)
					    (prev-step-matches restaurant 
							       '("ask-if-all")
							       2)))
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) "~%All right.")))) 



(defparameter end-scenario 
  (make-instance 'scenario
		 :name "end-scenario"
		 :description "End things."
		 :parent general-scenario
		 
		 :end-condition 
		 (orc 
		  (list 
		   (lambdac #'(lambda (restaurant mystep)

				(if (equal (name (nth-last-step 
						  (waiter restaurant) 1)) 
					   "get-item")
				    (make-instance 'results :success t)
				  (make-instance 'results :success nil)))) 
		   (lambdac #'(lambda (restaurant mystep)
                                 
				(if (equal (name (nth-last-step 
						  (waiter restaurant) 1)) 
					   "invalid-order")
				    (make-instance 'results :success t)
				  (make-instance 'results :success nil)))) 
		   (historyc "present-bill")
		   (historyc "yes-response")))))

(add-steps end-scenario 
	   (list ask-if-all present-bill bring-something-else yes-response))


(defparameter give-bill 
  (make-instance 'mystep
		 :name "give-bill"
		 :my-condition (andc :nec 
				     (list (notc (historyc "present-bill"))
					   (notc (historyc "give-bill"))
					   ))
		 :my-action #'(lambda (self restaurant)
				
				
				(say (waiter restaurant) "~%Here is your bill."
				     )
				(say (waiter restaurant) "~% ")

				(display-bill (order-pad restaurant))

				(say (waiter restaurant) 
				     "~%Your total comes to $~a"
				     (calculate-bill (order-pad restaurant)))
				)
		 ))

(defparameter say-good-bye 
  (make-instance 'mystep
		 :name "say-good-bye"
		 :my-condition (orc (list (historyc "present-bill")
					  (historyc "give-bill")
					  (historyc "bring-something-else")))
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
    "~%Thank-you for visiting Susie's Restaurant! Please come again sometime!")
				)
		 ))              
                 
(defparameter wrap-up 
  (make-instance 'scenario
		 :name "wrap-up"
		 :description "Give bill and say good-bye."
		 :steps (list give-bill say-good-bye)
		 :end-condition (historyc "say-good-bye")))
            







