;#####################################################################
;#                                                                   #
;#                 ORDRFORM.LSP                                      #
;#                                                                   #
;#   CONTENTS:                                                       #
;#      Classes:  order-form                                         #
;#      Comment:  This file contains the order-form class and its    #
;#                associated methods.                                #
;#                                                                   #
;#####################################################################


;******************************************************
;*                                                    *
;*               ORDER-FORM CLASS                     *
;*                                                    *
;* drinks-to-be-served, dessert-to-be-served and      *
;* food-to-be-served are counters that keep track of  *
;* how many of eack type of items have been ordered   *
;* but not served yet.  This will be used in the      *
;* serve scenario.                                    *
;* drink-items, main-course and dessert-items are     *
;* where the items are written when they are ordered. *
;******************************************************
(defclass order-form ()

   ((drinks-to-be-served :accessor num-drinks :initform 0)
   (food-to-be-served :accessor food :initform 0)
    (dessert-to-be-served :accessor num-dessert :initform 0)
    (num-items-ordered :accessor num-items :initform 0)
    (appetizers :accessor appetizers :initform '())
    (main-course :accessor main-course :initform '())
    (drink-items :accessor drink-items :initform '())
    (dessert-items :accessor dessert-items :initform '())
    (party-size :accessor party-size :initform 1)))

(defmethod display-bill ((self order-form))
;; This method print all of the items that have been ordered on the screen.

(format t "~%Drinks:")
   (loop for item in (drink-items self) do
         (if (not (kind-of (class-name (class-of item)) 'nothing))
             (format t "~%    ~a  $~a" (reference-name item) (cost item))))
(format t "~%Food:")
   (loop for item in (main-course self) do
         (if (not (kind-of (class-name (class-of item)) 'nothing))
             (format t "~%    ~a  $~a" (reference-name item) (cost item))))

   (loop for item in (dessert-items self) do
         (if (not (kind-of (class-name (class-of item)) 'nothing))
             (format t "~%    ~a  $~a" (reference-name item) (cost item))))

)


(defmethod increment-counters ((self order-form) item)
;; This increment the counter of the type of item by one.

(cond ((kind-of (class-name (class-of item)) 'drink-item)
        (setf (num-drinks self) (+ (num-drinks self) 1)))
((kind-of (class-name (class-of item)) 'dessert-item)
    (setf (num-dessert self) (+ (num-dessert self) 1)))
(t (setf (food self) (+ (food self) 1))))
(setf (num-items self) (+ (num-items self) 1)))


(defmethod decrement-counters ((self order-form) item)
;; This decrease the counter of the item by one.

(cond ((kind-of (class-name (class-of item)) 'drink-item)
        (setf (num-drinks self) (- (num-drinks self) 1)))
((kind-of (class-name (class-of item)) 'dessert-item)
    (setf (num-dessert self) (- (num-dessert self) 1)))
(t (setf (food self) (- (food self) 1))))
(setf (num-items self) (- (num-items self) 1)))


(defmethod calculate-bill ((self order-form))
  :documentation "Adds the cost of each item and returns the total.
   (No tax calculated.)" 
   (with-slots (appetizers main-course drink-items dessert-items) self
     (+  (loop for item in appetizers  sum (cost item))
      (loop for item in main-course  sum (cost item))
      (loop for item in drink-items sum (cost item))
      (loop for item in dessert-items sum (cost item))
)))

(defmethod new-bill ((self order-form))
  (setf (appetizers self) '())
  (setf (main-course self) '())
  (setf (drink-items self) '())
  (setf (dessert-items self) '())
  (setf (party-size self) 1))


(defmethod add-to-order-pad ((self order-form) item)
;; Adds the item to the correct section on the pad.
  (cond ((kind-of (class-name (class-of item)) 'drink-item) 
         (push item (drink-items self)))
        ((kind-of (class-name (class-of item)) 'dessert-item)
         (push item (dessert-items self)))
        (t (push item (main-course self)))))

(defmethod add-to-section ((self order-form) section item)
;; This adds the item to the specified section.

   (cond ((equal section "drink-items")
          (push item (drink-items self)))
         ((equal section "dessert-items")
           (push item (dessert-items self)))
         (t (push item (main-course self)))))

(defmethod remove-item ((self order-form) item)
;; This removes the item from the correct section.
  (when debugp (format t "~%    REMOVING ~a FROM A SECTION" item))
  (cond ((kind-of (class-name (class-of item)) 'drink-item)
            (delete item (drink-items self)))
        ((kind-of (class-name (class-of item)) 'dessert-item)
          (delete item (dessert-items self)))
        (t (delete item (main-course self))))
 (when debugp (format t "~%    NEW ORDER-FORM: ~a" self)))

(defmethod print-object ((self order-form) stream)
  (format stream "<order-form: ~a ~a  ~a.>" (drink-items self) 
	  (main-course self) (dessert-items self)))

(defmethod dump-object ((self order-form) stream)
   (format stream "~%Order Pad")
   (format stream "~%   Drinks")
   (loop for drink in (drink-items self) do
      (dump-object drink stream))
   (format stream "~%   Entrees")
   (loop for drink in (main-course self) do
      (dump-object drink stream))
    (loop for dessert in (dessert-items self) do
      (dump-object dessert stream)))






