;########################################################################
;#                                                                      #
;#                      OTHERGRAMMAR.LSP                                #
;#                                                                      #
;#  CONTENTS:                                                           #
;#     Classes:  noun-adj-error, what-that-error, french-error,         #
;#               sample-word, french-word, french-phrase,               #
;#               direct-translation-error, adj-and-adj-error,           #
;#               idiom-error, idiom, infinitive                         #
;#    Functions:  parse-from-sample-dictionary (input-line)             #
;#                noun-adj (input-line input-line-num)                  #
;#                look-for-adj-and-adj (input-line input-line-num)      #
;#                find-want-that (input-line input-line-num)            #
;#                look-for-french-words (input-line input-line-number)  #
;#                look-for-bad-translations (input-line input-line-num) #
;#                idiom-could-be-used (input-line input-line-num)       #
;#                look-for-infinitives (input-line input-line-num)      #
;#    Comment:  This file is also used in the post-dialogue analysis.   #
;#                                                                      #
;########################################################################
            


;*********************************************************
;*                                                       *
;*        NOUN-ADJ-ERROR CLASS                           *
;*                                                       *
;*  This class is designed to hold individual mistakes   *
;* where the adj follows a noun.                         *
;*                                                       *
;*********************************************************

(defclass noun-adj-error ()
  ((noun
    :initarg :noun
    :accessor noun
    :initform "")
   (adj
    :initarg :adj
    :accessor adj
    :initform "")
   (line-num
    :initarg :line-num
    :accessor line-num
    :initform 0)))


;*********************************************************
;*                                                       *
;*         WANT-THAT-ERROR CLASS                         *
;*                                                       *
;*  Person speaking people have a tendancy to follow     *
;* the verbe want by a that clause.  The class is        *
;* designed to hold errors of this type.                 *
;*                                                       *
;*********************************************************

(defclass want-that-error ()
  ((line-num
    :initarg :line-num
    :accessor line-num
    :initform 0)
   (message
    :initarg :message
    :accessor message
    :initform "The verb WANT should not be followed by a THAT clause.")))


;***********************************************************
;*                                                         *
;*              FRENCH-ERROR CLASS                         *
;*                                                         *
;*   This class is designed to hold errors where the user  *
;* has used a french word rather than english word.        *
;*                                                         *
;***********************************************************

(defclass french-error ()
  ((french-used
    :initarg :french-used
    :accessor french-used
    :initform "")
   (english-form
    :initarg :english-form
    :accessor english-form
    :initform "")
   (line-num
    :initarg :line-num
    :accessor line-num
    :initform 0)))
    
;***************************************************************
;*                                                             *
;*                 SAMPLE-WORD CLASS                           *
;*                                                             *
;*  This class represent a single word in the sample           *
;* dictionary.  Each instance contains that actually word and  *
;* it's syntactic category.                                    *
;*                                                             *
;***************************************************************


(defclass sample-word ()
  ( (word 
     :initarg :word
     :accessor word
     :initform ""
     )
    (tense
     :initarg :tense
     :accessor tense
     :initform "")
    (cat
     :initarg :cat
     :accessor cat
     :initform "")))


(defparameter sample-dictionary '())

(setf not-in-sample-dictionary (make-instance 'sample-word
					      :word "not-in-dictionary"
					      :cat ""))

(setf sample-dictionary 
      (make-instance 'dictionary
		     :contents (list
		;nouns
				(make-instance 'sample-word
					       :word "coffee"
					       :cat "noun")
                                (make-instance 'sample-word
                                               :word "tea"
                                               :cat "noun")
				(make-instance 'sample-word
					       :word "coke"
					       :cat "noun")
				(make-instance 'sample-word
					       :word "soft-drink"
					       :cat "noun"
					       )
				(make-instance 'sample-word
					       :word "fries"
					       :cat "noun")
				(make-instance 'sample-word
					       :word "steak"
					       :cat "noun"
					       )
				(make-instance 'sample-word
					       :word "pasta"
					       :cat "noun")
				
          
		;adjectives
				(make-instance 'sample-word
					       :word "large"
					       :cat "adj")
				(make-instance 'sample-word
					       :word "small"
					       :cat "adj")
				(make-instance 'sample-word
					       :word "hot"
					       :cat "adj")
				(make-instance 'sample-word
					       :word "cold"
					       :cat "adj")
		 ;pronouns
				(make-instance 'sample-word
					       :word "that"
					       :cat "pronoun")

                 ;conjunctions
				(make-instance 'sample-word
                                               :word "and"
					       :cat "conjunction")
		  ;verbs
				(make-instance 'sample-word
					       :word "want"
					       :cat "verb"
                                               :tense "infinitive")  
				(make-instance 'sample-word
					       :word "be"
					       :cat "verb"
					       :tense "infinitive")
                                (make-instance 'sample-word
					       :word "will"
					       :cat "verb"
					       :tense "infinitive") 
				(make-instance 'sample-word
                                                :word "have"
                                                :cat "verb"
                                                :tense "infinitive")
				)))


(defparameter french-dictionary '())


;*******************************************************
;*                                                     *
;*          FRENCH-WORD CLASS                          *
;*                                                     *
;*  This class represent a word in the french          *
;* dictionary.  Each instance contains the french form *
;* and the english form of the word.                   *
;*                                                     *
;*******************************************************

(defclass french-word ()
  ((french
    :initarg :french
    :accessor french
    :initform "")
   (english 
    :initarg :english
    :accessor english
    :initform "")))

(setf french-dictionary 
      (make-instance 'dictionary
		     :contents 
		     (list
		      (make-instance 'french-word
				     :french "boisson"
				     :english "drink")
		      (make-instance 'french-word
				     :french "je"
				     :english "I")
		      (make-instance 'french-word
				     :french "cafe"
				     :english "coffee")
		      (make-instance 'french-word
				     :french "eau"
				     :english "water")
		      (make-instance 'french-word
				     :french "lait"
				     :english "milk")
		      (make-instance 'french-word
				     :french "gateau"
				     :english "cake")
		      (make-instance 'french-word
				     :french "bifteck"
				     :english "steak")
		      (make-instance 'french-word
				     :french "poulet"
				     :english "chicken"))))


;********************************************************
;*                                                      *
;*       FRENCH-PHRASE CLASS                            *
;*                                                      *
;*  This class is designed to hold a french phrase      *
;* it's direct translations and the correct             *
;* translation.                                         *
;*                                                      *
;********************************************************

(defclass french-phrase ()
  ((french-form
    :initarg :french-form
    :accessor french-form
    :initform "")
   (possible-direct-translations
    :initarg :possible-direct-translations
    :accessor possible-direct-translations
    :initform '())
   (correct-translation
    :initarg :correct-translation
    :accessor correct-translation
    :initform "")))

(defparameter french-phrase-dictionary '())

(setf french-phrase-dictionary 
      (make-instance 'dictionary
		     :contents 
		     (list
		      (make-instance 'french-phrase
				     :french-form "quel est le prix"
				     :possible-direct-translations 
				     '("what is the price" "what is the cost")
				     :correct-translation "how much is")
		      (make-instance 'french-phrase
				     :french-form "l'addition s'il vous plait"
				     :possible-direct-translations 
				     '("the addition please" 
				       "could I have the addition please" 
				       "could I have the addition")
				     :correct-translation 
				     "Could I have the bill please?")
		      )))

;************************************************************
;*                                                          *
;*         DIRECT-TRANSLATION-ERROR CLASS                   *
;*                                                          *
;*  This class is designed to hold errors where the user    *
;* has tried to directly translated something from french   *
;* to english.                                              *
;*                                                          *
;************************************************************


(defclass direct-translation-error ()
  ((french-phrase
    :initarg :french-phrase
    :accessor french-phrase
    :initform "")
   (translation-given
    :initarg :translation-given
    :accessor translation-given
    :initform "")
   (correct-translation
    :initarg :correct-translation
    :accessor correct-translation
    :initform "")
   (line-num
    :initarg :line-num
    :accessor line-num
    :initform 0)))


;*************************************************************
;*                                                           *
;*             ADJ-AND-ADJ-ERROR CLASS                       *
;*                                                           *
;*  This class is designed to keep tract of places where     *
;* the user is placed the conjunction "and" between two      *
;* adjectives.                                               *
;*                                                           *
;*************************************************************



(defclass adj-and-adj-error ()
  ((1st-adj
    :initarg :1st-adj
    :accessor 1st-adj
    :initform "")
   (2nd-adj
    :initarg :2nd-adj
    :accessor 2nd-adj
    :initform "")
   (line-num
    :initarg :line-num
    :accessor line-num
    :initform 0)))


;****************************************************************
;*                                                              *
;*                                                              *
;*                    IDIOM-ERROR CLASS                         *
;*                                                              *
;*  This class is designed to hold instance where the user      *
;* could have used an idiom and didn't.                         *
;*                                                              *
;****************************************************************


(defclass idiom-error ()
  ((actual-idom
    :initarg :actual-idiom
    :accessor actual-idiom
    :initform ""
    )
   (phrase-given
    :initarg :phrase-given
    :accessor phrase-given
    :initform "")
   (line-num
    :initarg :line-num
    :accessor line-num
    :initform 0)))


;****************************************************************
;*                                                              *
;*               IDIOM CLASS                                    *
;*                                                              *
;*  This instances of this class contain an idiom and a set of  *
;*  phrases that could be employed instead.                     *
;*                                                              *
;****************************************************************


(defclass idiom ()
  ((actual-idiom
    :initarg :actual-idiom
    :accessor actual-idiom
    :initform "")
   (other-phrases
    :initarg :other-phrases
    :accessor other-phrases
    :initform "")))


(defparameter idiom-dictionary 
  (make-instance 'dictionary
		 :contents 
		 (list
		  (make-instance 'idiom
				 :actual-idiom "black"
				 :other-phrases '("without cream or sugar" 
						  "no cream and no sugar" 
					     "without cream and without sugar"
					     )))))





;*****************************************************************
;*                                                               *
;*                   INFINITIVE CLASS                            *
;*                                                               *
;*  This class is designed to hold instance where the user uses  *
;* the inifinitive form of a verb.                               *
;*                                                               *
;*****************************************************************


(defclass infinitive ()
  ((verb
    :initarg :verb
    :accessor verb
    :initform "")
   (line-num
    :initarg :line-num
    :accessor line-num
    :initform 0)))
              
;*****************************************************
;;  FUNCTIONS USED TO FIND ERRORS
           

(defun parse-from-sample-dictionary (input-line)
;; This function examins each word entered by the user and tries
;; to match it with a word in the sample dictionary.  If no match
;; is found, "not-in-dictionary" is added to the parsed-input.
  (let* ((input-to-check (walk-string
			  recognize-words (string-downcase input-line)))
	 (parsed-input '())
	 (results '())
	 (substring '())
	 (count 0)
	 
	 (noun "")
	 (found-word '())
	 (ajd "")
	 (index 0)
	 
	 (end-condition (length input-to-check)))

    (loop until (eq index end-condition) do

	  (setf substring (nth-cdr index input-to-check))
	  (loop for a-word in (contents sample-dictionary)  until found-word do
 
		(if (lines-up (walk-string recognize-words
					   (string-downcase (word a-word)))
			      substring)
		    (setf found-word a-word))
  
		)
	  (cond (found-word
		 (setf parsed-input (push found-word parsed-input)))
		(t (setf parsed-input (push not-in-sample-dictionary 
					    parsed-input))))
	  (setf found-word '())
	  (incf index 1))
    
    (setf parsed-input (reverse parsed-input))
    
    
    parsed-input))

;;Function to find adjective after noun errors:
;**************************************************


(defun noun-adj (input-line input-line-num)
;; This function looks for nouns in the parsed input.  If a noun is found
;; it check if if the next word is an adjective.  If so, an instance of
;; the noun-adj-error class is created and added to noun-adj section of
;; the list of errors.

  (let* ((parsed-input (parse-from-sample-dictionary input-line))
	 (index 0)
	 (noun-index-list '())
	 (next-word '())
	 (temp '()))
    
    (loop for a-word in parsed-input do
	  (if (equal (cat a-word) "noun")
	      (push index noun-index-list))
	  (incf index 1))
    
;;(format t "~%Index list: ~a" noun-index-list)
;;(format t "~%Length of Parsed Input: ~a" (length parsed-input))



    (loop for an-index in noun-index-list do
	  (cond ((<= an-index (- (length parsed-input) 2))
		 (setf next-word (nth (+ 1 an-index) parsed-input))
		     
		 (cond ((equal (cat next-word) "adj")
			(setf temp (make-instance 'noun-adj-error
						  :noun (word 
							 (nth an-index 
							      parsed-input))
						  :adj (word next-word)
						  :line-num input-line-num))
			(setf (noun-adjective list-of-errors) 
			      (push temp (noun-adjective list-of-errors)))
			))
		 ))
	  
	  )))


;  Function to find the conjunction "and" between two adjectives
;*****************************************************************


(defun look-for-adj-and-adj (input-line input-line-num)
;;  This function looks for places where "and" is in between two adjective.
;; If such an occurrence is found, an instance of the adj-and-adj-error class
;; is created and added to the adj-and-adj section of the list-of-errors.

  (let* ((parsed-input (parse-from-sample-dictionary input-line))
	 (index 0)
	 (adj-index-list '())
	 (1st-next-word '())
         (2nd-next-word '())
	 (temp '()))

    (loop for a-word in parsed-input do
	  (if (equal (cat a-word) "adj")
	      (push index adj-index-list))
	  (incf index 1))
    
;;(format t "~%Index list: ~a" adj-index-list)
;;(format t "~%Length of Parsed Input: ~a" (length parsed-input))



    (loop for an-index in adj-index-list do
	  (cond ((<= an-index (- (length parsed-input) 3))
		 (setf 1st-next-word (nth (+ 1 an-index) parsed-input))
                 (setf 2nd-next-word (nth (+ 2 an-index) parsed-input))
              
                		     
		 (cond ((and (equal (word 1st-next-word) "and") 
			     (equal (cat 2nd-next-word) "adj"))
                
			(setf temp (make-instance 'adj-and-adj-error
						  :1st-adj (word 
							    (nth an-index
								 parsed-input))
						  :2nd-adj (word 2nd-next-word)
						  :line-num input-line-num))
			(setf (adj-and-adj list-of-errors) 
			      (push temp (adj-and-adj list-of-errors)))
                
			))
		 ))
	  
	  )))





; Function to find want that errors:
;***************************************************

(defun find-want-that (input-line input-line-num)
;;  This function looks for the verbe want in the parsed-input.  If want is
;; found, it checks if it is followed by "that".  If so, an instance of the
;; wnat-that-error class is created and added to the want-that section
;; of the list-of-errors.

  (let* ((parsed-input (parse-from-sample-dictionary input-line))
	 (index 0)
	 (want-index-list '())
	 (next-word '())
	 (temp '()))


    (loop for a-word in parsed-input do
	  (if (equal (word a-word) "want")
	      (push index want-index-list))
	  (incf index 1))
    
    ;;(format t "~%Index list: ~a" want-index-list)


    (loop for an-index in want-index-list do
	  (cond ((<= an-index (- (length parsed-input) 2)) 
		 (setf next-word (nth (+ 1 an-index) parsed-input))
		 
		 (cond ((equal (word next-word) "that")
			(setf temp (make-instance 'want-that-error
						  :line-num input-line-num))
			(setf (want-that list-of-errors) 
			      (push temp (want-that list-of-errors)))
			
			))
		 ))
	  
	  )))


;Function to find french words:
;*****************************************************          

(defun look-for-french-words (input-line input-line-number)
;; This function looks for french words in the user's input.  When a french
;; word is found, an instance of the french-error class of created and added
;; to the french-word-choice section of the list-of-errors.

  (let* ((input-to-check (walk-string
			  recognize-words
                            (string-downcase input-line)))
	 (found-word '())
	 (temp '())
	 (substring '())
	 (index 0)
	 (end-condition (length input-to-check)))
    
    (loop until (eq index end-condition) do
	  
	  (setf substring (nth-cdr index input-to-check))
	  (loop for a-word in (contents french-dictionary) until found-word do
		
		(if (lines-up (walk-string recognize-words
					   (string-downcase (french a-word))) 
			      substring)
		    (setf found-word a-word))
		
		)
	  
	  (cond (found-word
		 (setf temp (make-instance 'french-error
					   :french-used (french found-word)
					   :english-form (english found-word)
					   :line-num input-line-number))
		 
		 (setf (french-word-choice list-of-errors) 
		       (push temp (french-word-choice list-of-errors)))))
	
	  (setf found-word '())
	  (incf index 1)
	  )))


;Function to look for direct translations:
;*********************************************************

(defun look-for-bad-translations (input-line input-line-num)
;;  This function looks for occurances of common inappropriate direct 
;; translation that have been listed in the french-phrase-dictionary.
;; If any are found, instances of the direct-translation-error class
;; are created and added to the direct-translation section of the 
;; list-of-errors
  (let* ((input-to-check (walk-string recognize-words 
				      (string-downcase input-line)))
	 (found-phrase '())
	 (temp '())
	 (substring '())
	 (index 0)
	 (end-condition (length input-to-check)))
    
    (loop until (eq index end-condition) do
	  
	  (setf substring (nth-cdr index input-to-check))
	  
	  (loop for a-phrase in (contents french-phrase-dictionary) 
		until found-phrase do
  
		(loop for a-possibility in 
		      (possible-direct-translations a-phrase) 
		      until found-phrase do
		      (setf found-phrase (lines-up 
					  (walk-string recognize-words
						       (string-downcase 
							a-possibility)) 
					  substring)
			    )
		      (cond (found-phrase 
			     (setf temp 
				   (make-instance 'direct-translation-error
						  :translation-given 
						  a-possibility
						  :correct-translation 
						  (correct-translation a-phrase
								       )
						  :french-phrase (french-form 
								  a-phrase)
						  :line-num input-line-num))
			     (setf (direct-translation list-of-errors) 
				   (push temp (direct-translation 
					       list-of-errors)))
			     
			     )))
		)
	  
	  (setf found-phrase '())
	  
	  (cond (temp
		 (incf index (length (walk-string recognize-words 
						  (string-downcase 
						   (translation-given temp)))))
		 (setf temp '()))
		(t (incf index 1)))
	  ))
  )
        


;Function to look for places where idioms could be used:
;****************************************************


(defun idiom-could-be-used (input-line input-line-num)
;;  This function looks for places where an English idiom could have
;; been used and wasn't.  If such an occurrence is found, an instance
;; of the idiom-error class is created and added to the idioms section
;; of the list-of-errors.

(let* ((input-to-check (walk-string recognize-words 
				    (string-downcase input-line)))
       (found-idiom '())
       (temp '())
       (substring '())
       (index 0)
       (end-condition (length input-to-check)))
    
  (loop until (eq index end-condition) do
	
	(setf substring (nth-cdr index input-to-check))
	  
	(loop for an-idiom in (contents idiom-dictionary) until found-idiom do
  
	      (loop for a-possibility in (other-phrases an-idiom) 
		    until found-idiom do
		    (setf found-idiom (lines-up (walk-string recognize-words
							     (string-downcase 
							      a-possibility)) 
						substring)
			  )
		    (cond (found-idiom 
			     (setf temp 
				   (make-instance 'idiom-error
						  :phrase-given a-possibility
						  :actual-idiom (actual-idiom 
								 an-idiom)
						  :line-num input-line-num))
			     (setf (idioms list-of-errors) 
				   (push temp (idioms list-of-errors)))

			     )))
	      )
	
	(setf found-idiom '())

	(cond (temp
	       (incf index (length (walk-string recognize-words 
						(string-downcase (phrase-given
								  temp)))))
	       (setf temp '()))
	      (t (incf index 1)))
	))
)


        
;  Function to look for infinitives
;****************************************************************

(defun look-for-infinitives (input-line input-line-num)


  (let* ((parsed-input (parse-from-sample-dictionary input-line))
	 (index 0)
	 (verb-index-list '())
	 
	 (temp '()))


    (loop for a-word in parsed-input do
	  (if (equal (cat a-word) "verb")
	      (push index verb-index-list))
	  (incf index 1))

    ;;(format t "~%Index list: ~a" verb-index-list)


    (loop for an-index in verb-index-list do
	  (cond ((equal (tense (nth an-index parsed-input)) "infinitive")
		 (setf temp 
		       (make-instance 'infinitive
				      :verb (word (nth an-index parsed-input))
				      :line-num input-line-num))
		 (setf (infinitive-used list-of-errors) 
		       (push temp (infinitive-used list-of-errors)))
		 )))
    ))
	  
	  

             
      









