;###########################################################################
;#                                                                         #
;#                                                                         #
;#                        PHRASEC.LSP                                      #
;#                                                                         #
;#   CONTENTS:                                                             #
;#       classes:  phrase-cond, word-features, wild-card                   #
;#       functions:  phrased (necs)                                        #
;#       parameters:  indef-det, def-det, det                              #
;#       Comment:  Phrase-condition are only used in the get-item steps    #
;#                                                                         #
;###########################################################################

(defclass phrase-cond (step-condition)())

(defmethod do-test ((self phrase-cond) restaurant mystep)
     :documentation
   " Phrasec can take as arguments: 
     - symbols (or strings?) that become words
     - other phrasec's and orc's
     - the :nec keyword
     - the :free keyword, which means that the next item can appear anywhere 
        after this point. 
       The 'item' can be anything else that appears in a phrasec.
     - object-classes (for finding if the object of a word belongs to a 
       certain class of objects)
.    - word-classes (for finding a word that shares features with some other 
      words, eg. 
                       indefinite determiner)
.    - wild cards (to say how many words can intervene)
     - pairs:                      } 
          (object-class variable)  } These are used to collect bindings and 
                                   attach them to a
          (phrasec   variable)     } user-defined variable.
         (could make these a kind of class?)
.    - disjunctions of words eg. '(want have give)= 'one of these words:'"  
   (test-rest self restaurant mystep 0 (input (waiter restaurant)) 
	      (necessary-conditions self) nil))

   
(defmethod test-rest ((self phrase-cond) restaurant mystep count input 
		      remaining-arguments necessary)   
   (let ((more-results '())
         (current-results '())
         (old-input (input (waiter restaurant)))
         (argument (car remaining-arguments)))
      
      (when debugq (format t "~%Input is:    ~a" input)
         (format t "~%looking for: ~a" argument)
         (format t "~% COUNT IS: ~a" count))
      
      (cond ((null argument) 
             (when debugq (format t "~%success!"))
             (make-instance 'results :success t :remaining-input input 
			    :score nil))
            ((eq argument :nec)                                              
;The :nec keyword
             (when debugq (format t "~%argument was a :nec"))
             (test-rest self restaurant mystep count input 
			(cdr remaining-arguments) t))
            ((eq argument :free)
             (when debugq (format t "~%argument was a :free"))               
;The :free keyword
             (setf argument (second remaining-arguments))
             (let ((found nil))
                (loop for index from 0 below (length input)
                   until found do
                   (setf current-results
                         (test-rest self restaurant mystep
                            (+ count index) (nth-cdr index input) 
			    (list argument) nil))
                   (when current-results
                      (setf more-results (test-rest self restaurant mystep
                                            count  input 
					    (cddr remaining-arguments)  nil))
                      (when more-results 
     ;(when more-results uses some of the same input items as current-results,
     ;then fail. Otherwise, might as well call it a success.
                         (unless (intersection (flatten 
						(score current-results)) 
                                    (flatten (score more-results)))
                            (setf found t)
                            (add-to-score more-results (score current-results))
                            (add-bindings more-results 
					  (bindings current-results))
                            (setf (remaining-input more-results) 
                                  (remove-words more-results 
                                   (flatten (score current-results))
                                   count))))
                      )
                   )
                (if found 
                   more-results
                   (if necessary 
                      (make-instance 'results :success nil :score '())
                      (test-rest self restaurant mystep count input 
				 (cddr remaining-arguments) nil))))
                ) 
            ((typep argument 'wild-card)                                    
        ;Wildcards (w/max & min) 
 ;;;With a wild card what we want to do is to start with the minimum number of 
;;; wild-things. "eat up" that many tokens from the input, then check if 
 ;;; the rest of the input is parseable. 
;;;  If not, "eat up" one more token from the 
;;; input and check again, until either the input is parseable or we've 
;;; reached the 
             ;;; maximum allowed wild-things or there is no more input.
             (when debugq (format t "~%argument was a wildcard"))
             (loop for i from (minimum argument) to (maximum argument) do
                (when debugq (format t "~%   ~ath iteration in wildcard: 
(~a ~a)" i
                                (minimum argument) (maximum argument)))
                (setf more-results
                      (test-rest self restaurant mystep
                         (+ count i) (nth-cdr i input) 
(cdr remaining-arguments) nil))
                until (success more-results))
             (when debugq (format t "~%wildcard (~a ~a) succeeded" 
                (minimum argument) (maximum argument)))
             more-results)
            ((typep argument 'step-condition)                             
;orc, andc, phrasec  
;;;NOTE: NOTC WORKS TOO, BUT IS VERY VERY TRICKY TO USE, 
;;;ESPECIALLY WHEN WILDCARDS 
;;; OCCUR ANYWHERE IN THE PHRASEC. THE WORST SITUATION SEEMS TO BE 
;;; (wildcard)-(not-phrasec).
;;; IT WORKS BETTER WHEN YOU HAVE not-(phrasec-wildcard), 
;;; WHERE THE WILDCARD IS INSIDE 
             ;;; THE PHRASEC.
             (when debugq (format t "~%argument was a step-condition"))
             (setf (input (waiter restaurant)) input)
             (setf current-results (apply #'do-test 
(list argument restaurant mystep)))
             (setf (input (waiter restaurant)) old-input)
             (when debugq 
(format t "~%RESULTS FROM STEP-CONDITION: ~a" current-results))
             (cond ((success current-results)
          ;;count <- count + the difference between the current input and the 
         ;; remaining-input of current-results.
 ;;add-to-score will combine the results of current-results to the results of 
 ;; more-results, as achieved by applying the test-rest.
                    (when (typep argument 'not-cond)
                       (when debugq 
                          (format t 
            "~%Step-condition was a not. setting input of results to: ~a" 
                             input))
                       (setf (remaining-input current-results) input))
                    (setf (score current-results) (flatten 
						   (score current-results)))
                    (setf (score current-results) 
                          (loop for i in (score current-results) collect 
				(+ i count)))
                    (when debugq 
			  (format t "~%setting count to ~a for more-results" 
                       (+ count (- (length input) 
				   (length 
				    (remaining-input current-results))))))
                    (setf more-results
                          (test-rest self restaurant mystep
                             (+ count (- (length input) 
                                         (length 
					  (remaining-input current-results))))
                             (remaining-input current-results)
                             (cdr remaining-arguments)
                             nil))
                    (add-bindings more-results (bindings current-results))
                    (add-to-score more-results (score current-results))
                    more-results)
                   (t (if necessary 
                         (make-instance 'results :success nil :score nil)
                         (test-rest self restaurant mystep count input 
				    (cdr remaining-arguments) nil)))))
            ((typep argument 'pair)                                       
;pairs: (step-cond var)
             (let ((head (head argument))                                 
;or (object-type var)
                   (var  (tail argument)))
                (when debugq (format t "~%argument was a pair"))
                (cond ((typep head 'step-condition) 
                       (setf (input (waiter restaurant)) input)
                       (setf current-results 
			     (apply #'do-test (list head restaurant mystep)))
                       (setf (input (waiter restaurant)) old-input)
                       (when debugq 
			     (format t 
				     "~%RESULTS FROM PAIR: ~a" 
				     current-results))
                       (cond ((success current-results)
                              (when debugq 
				    (format t "~%making a binding: ~a ~a" 
                                              var (bindings current-results)))
                              (when (typep head 'not-cond)
                                 (setf 
				  (remaining-input current-results) input))
          ;;;Doing a do-list resets the count variable; we need to add in the 
                              ;;; current count to make them match.
                              (setf (score current-results) 
				    (flatten (score current-results)))
                              (setf (score current-results) 
                                 (loop for i in (score current-results) collect
                                       (+ i count)))
                              (when debugq 
				    (format t 
				      "~%setting count to ~a for more-results" 
                                (+ count (- (length input) 
					    (length (remaining-input 
						     current-results))))))
                              (setf more-results
                                    (test-rest self restaurant mystep
                                      (+ count (- (length input)
                                                  (length 
						   (remaining-input 
						    current-results))))
                                       (remaining-input current-results)
                                       (cdr remaining-arguments)
                                       nil))
                              (add-bindings more-results (list 
							  (list var 
								(bindings 
							    current-results))))
                              (add-to-score more-results 
					    (score current-results))
                              more-results)
                             (t (if necessary 
                                   (make-instance 'results 
						  :success nil 
						  :score nil)
                                   (test-rest self restaurant mystep count 
					      input 
                                      (cdr remaining-arguments) nil)))))
                      (t (setf current-results 
			       (test-rest self restaurant mystep count
					  input (list head) t))
                         (cond ((success current-results)
                                  (setf more-results 
					(test-rest self restaurant mystep
                                                      (+ count 
							 (- 
							  (length input)
							  (length 
							   (remaining-input 
							    current-results))))
                                                      (remaining-input 
						       current-results)
                                                      (cdr remaining-arguments)
                                                       nil))
                                  (add-to-score more-results (list count))
                                  (add-bindings more-results 
						(list (list var (car input) )))
                                  more-results)
			       (t (if necessary 
				      (make-instance 'results 
						     :success nil :score nil)
				    (test-rest self 
					       restaurant mystep count input 
					       (cdr remaining-arguments) nil))
				  ))))))
              ((listp argument)                                           
					;'(have want get) 
(when debugq (format t "~%argument was a list"))
(let ((found nil))
  (loop for wd in argument 
	until found do
	(setf current-results (fw wd input))
	(when current-results
	      (setf more-results (test-rest self restaurant mystep
					    (+ count 1)
					    (cdr input)
					    (cdr remaining-arguments)
					    nil))
                         (add-to-score more-results (list count))
                         (setf found t)))
                   (cond (found more-results)
                         (necessary  ;;and we didn't find it
                          (make-instance 'results :success nil :score nil))
                         (t (test-rest self restaurant mystep 
                               count input (cdr remaining-arguments) nil)))))
               ((typep argument 'word-features)
                                    ;(indef.det.)
                (when debugq (format t "~%  comparing ~a and ~a" argument 
				     (car input)))
                (setf current-results (compare argument (car input)))
                (when debugq (format t "~%   found: ~a" current-results))
                (cond (current-results
                       (setf more-results (test-rest self restaurant mystep
                                           (+ count 1)
                                           (cdr input)
                                           (cdr remaining-arguments)
                                           nil))
                       (add-to-score more-results (list count))
                       more-results)
                      (t (if necessary (make-instance 'results 
						      :success nil :score nil)
                            (test-rest self restaurant mystep count input 
				       (cdr remaining-arguments) nil))) ))
                (t 
                   (when debugq 
		    (format t "~%argument is a single word or object-class"))
                     (setf current-results (fw argument input))       
              ;symbols (wds and 
                   (when debugq (format t "~% fw: ~a" current-results))
                  (cond (current-results    
                                    ;obj classes that 
                         (setf more-results 
			       (test-rest self restaurant mystep     
					;don't have a binding)
                          (+ count 1)
                          (cdr input)
                          (cdr remaining-arguments)
                          nil))
                         (add-to-score more-results (list count))
                         more-results)
                        (t (if necessary 
                              (make-instance 'results :success nil :score nil)
                              (test-rest self restaurant mystep
                                 count input (cdr remaining-arguments) nil)))))
                  
               )))

(defun phrasec (necs)
   (make-instance 'phrase-cond :necessary-conditions necs))
    
                               
           

(defclass word-features ()
    ((class 
        :initarg :class
        :accessor class 
        :initform nil)
     (features 
        :initarg :features
        :accessor features
        :initform '())))

(defmethod compare ((self word-features) input-word)
   (let ((matches t))
      (when (class self)
         (unless (kind-of (class-name (class-of input-word)) 
                    (class self))
            (setf matches nil)))
      (loop for feature in (features self) until (not matches) do 
         (unless (equal (slot-value input-word (first feature))
                    (second feature))
            (setf matches nil)))
      matches))
            
                                        
(setf indef-det (make-instance 'word-features
                 :class 'determiner
                 :features '((definite nil))))
(setf def-det (make-instance 'word-features
                 :class 'determiner
                 :features '((definite t))))
(setf det (make-instance 'word-features
             :class 'determiner))


    
(defclass wild-card ()
    ((minimum 
      :initarg :minimum
      :accessor minimum
      :initform 0)
     (maximum 
      :initarg :maximum
      :accessor maximum
      :initform 1)))

        
(defun wildcard (min max)
    (make-instance 'wild-card
       :minimum min
       :maximum max))

(setf anystring (wildcard 0 30))


;;;-----------------------------------------------------------------
;;; EXAMPLE 
;;;     this is a phrasec that recognizes things 
;;;     that look like "...I.. have/want/order... a/(nil) coffee..."
;;;     but not "I want to know where the coffee I ordered got to."
;;;-----------------------------------------------------------------
;;; (setf a  (phrasec
;;;                     (list anystring
;;;                        :nec 'customer
;;;                        (wildcard 0 1)
;;;                         '(want have order)
;;;                         (wildcard 0 2)
;;;                        :nec
;;;                        focus-np-det-nec
;;;                           )))
;;; (setf det-nec (phrasec 
;;;                           (list
;;;                              :nec
;;;                              (make-instance 'pair
;;;                                 :head det
;;;                                 :tail 'determiner))))
;;; 
;;; (setf focus-np-det-nec (phrasec 
;;;                       (list :nec det-nec
;;;                          (wildcard 0 6)
;;;                          :nec  focus-np)))
;;; 
;;; (setf focus-np (phrasec (list :nec 
;;;                            (make-instance 'pair
;;;                               :head 'orderable-item
;;;                               :tail 'focus))))
      
;;; (parse w "I want a coffee.")
;;;     <step-results: (T (0 1 3 3) (FOCUS ((THING-TO-ORDER #))) ((.)))>
;;; (parse w "I want a coffee now.")
;;;    <step-results: (T (0 1 3 3) (FOCUS ((THING-TO-ORDER #))) ((n o w) (.)))>
;;; (parse w "Give me a coffee.")
;;;     <step-results: (T (1 3 3) (FOCUS ((THING-TO-ORDER #))) ((.)))>
;;; (parse w "Gimme coffee")        
 ;;;"customer" is not seen ("gimme" not parsed to "give me")
;;;      <step-results: (NIL NIL NIL NIL)>
;;; (parse w "I want coffee.")
;;;      <step-results: (T (0 1 3 2) (FOCUS ((THING-TO-ORDER #))) ((.)))>
;;; (parse w "me want coffee")
;;;      <step-results: (T (0 1 3 2) (FOCUS ((THING-TO-ORDER #))) NIL)>
;;; (parse w "order me a cup of coffee please.")
;;;        <step-results: (T (1 4 5) (FOCUS ((THING-TO-ORDER #))) 
;;;(<WORD: PLEASE> (.)))>   
;;;The next case is one that we want to fail. 
;;;The problem is that the first wildcard absorbs 
;;; "the" and we go into the pair with only "coffee"
;;; (parse w "bring me the coffee")
;;;        <step-results: (T (1 4 3) (FOCUS ((THING-TO-ORDER #))) NIL)>  
;;; (parse w "bring me the cup of coffee please.")
;;;       <step-results: (NIL NIL NIL NIL)>          









